@@ -13,11 +13,13 @@ module Language.Haskell.Meta.Parse (
   parseExp,
   parseType,
   parseDecs,
+  parseDecsWithMode,
   myDefaultParseMode,
   myDefaultExtensions,
   parseResultToEither,
   parseHsModule,
   parseHsDecls,
+  parseHsDeclsWithMode,
   parseHsType,
   parseHsExp,
   parseHsPat,
@@ -63,19 +65,20 @@ parseType = either Left (Right . toType) . parseHsType
 parseDecs :: String -> Either String [Dec]
 parseDecs  = either Left (Right . toDecs) . parseHsDecls
 
+-- | @since 0.8.2
+parseDecsWithMode :: ParseMode -> String -> Either String [Dec]
+parseDecsWithMode parseMode = either Left (Right . toDecs)
+  . parseHsDeclsWithMode parseMode
+
 -----------------------------------------------------------------------------
 
 {-# DEPRECATED myDefaultParseMode, myDefaultExtensions
   "The provided ParseModes aren't very meaningful, use your own instead" #-}
 myDefaultParseMode :: ParseMode
-myDefaultParseMode = ParseMode
+myDefaultParseMode = defaultParseMode
   {parseFilename = []
   ,baseLanguage = Haskell2010
   ,extensions = map EnableExtension myDefaultExtensions
-  ,ignoreLinePragmas = False
-  ,ignoreLanguagePragmas = False
-  ,fixities = Nothing
-  ,ignoreFunctionArity = False
   }
 
 myDefaultExtensions :: [KnownExtension]
@@ -103,6 +106,11 @@ parseHsDecls :: String -> Either String [Hs.Decl Hs.SrcSpanInfo]
 parseHsDecls = either Left (Right . moduleDecls)
   . parseResultToEither . parseModuleWithMode myDefaultParseMode
 
+-- | @since 0.8.2
+parseHsDeclsWithMode :: ParseMode -> String -> Either String [Hs.Decl Hs.SrcSpanInfo]
+parseHsDeclsWithMode parseMode = either Left (Right . moduleDecls)
+  . parseResultToEither . parseModuleWithMode parseMode
+
 
 parseHsType :: String -> Either String (Hs.Type Hs.SrcSpanInfo)
 parseHsType = parseResultToEither . parseTypeWithMode myDefaultParseMode
