@@ -135,7 +135,7 @@ instance ToName (Hs.SpecialCon l) where
   toName (Hs.UnitCon _) = mkName "()"
   toName (Hs.ListCon _) = ''[] -- Parser only uses this in types
   toName (Hs.FunCon _)  = ''(->)
-  toName (Hs.TupleCon _ _ n) = 
+  toName (Hs.TupleCon _ _ n) =
      mkName $ concat ["(",replicate (n-1) ',',")"]
   toName (Hs.Cons _)    = '(:)
 
@@ -236,6 +236,11 @@ instance ToExp (Hs.Exp l) where
   toExp (Hs.Con _ n)                 = ConE (toName n)
   toExp (Hs.Lit _ l)                 = LitE (toLit l)
   toExp (Hs.InfixApp _ e o f)        = UInfixE (toExp e) (toExp o) (toExp f)
+#if MIN_VERSION_template_haskell(2,12,0)
+  toExp (Hs.App _ e (Hs.TypeApp _ t)) = AppTypeE (toExp e) (toType t)
+#else
+  toExp (Hs.App _ e aTypeApp@Hs.TypeApp{}) = noTHyet "toExp" "2.12.0" aTypeApp
+#endif
   toExp (Hs.App _ e f)               = AppE (toExp e) (toExp f)
   toExp (Hs.NegApp _ e)              = AppE (VarE 'negate) (toExp e)
   toExp (Hs.Lambda _ ps e)         = LamE (fmap toPat ps) (toExp e)
@@ -314,11 +319,13 @@ instance ToName TyVarBndr where
   toName (PlainTV n) = n
   toName (KindedTV n _) = n
 
+#if !MIN_VERSION_haskell_src_exts(1,21,0)
 instance ToType (Hs.Kind l) where
   toType (Hs.KindStar _) = StarT
   toType (Hs.KindFn _ k1 k2) = toType k1 .->. toType k2
   toType (Hs.KindParen _ kp) = toType kp
   toType (Hs.KindVar _ n) = VarT (toName n)
+#endif
 
 toKind :: Hs.Kind l -> Kind
 toKind = toType
@@ -714,7 +721,7 @@ instance ToDecs (Hs.InstDecl l) where
 
 instance ToDecs (Hs.Decl l) where
   toDecs a@(Hs.TypeSig _ ns t)
-    = let xs = fmap (flip SigD (fixForall $ toType t) . toName) ns
+    = let xs = fmap (flip SigD (toType t) . toName) ns
        in xs
 
   toDecs (Hs.InfixDecl l assoc Nothing ops) =
@@ -729,17 +736,6 @@ instance ToDecs (Hs.Decl l) where
 
   toDecs a = [toDec a]
 
-collectVars e = case e of
-  VarT n -> [PlainTV n]
-  AppT t1 t2 -> nub $ collectVars t1 ++ collectVars t2
-  ForallT ns _ t -> collectVars t \\ ns
-  _          -> []
-
-fixForall t = case vs of
-  [] -> t
-  _  -> ForallT vs [] t
-  where vs = collectVars t
-
 instance ToDecs a => ToDecs [a] where
   toDecs a = concatMap toDecs a
 
